# meds/bots/__init__.py
#
#

"""
    A Bot can connect to a network and communicate with that network.
    Methods are available to join channels and announce text to those channels.

"""

from meds.utils.trace import get_exception
from meds.utils.name import sname

from meds.scheduler import Scheduler
from meds.launcher import Launcher
from meds.log import YELLOW, ENDC
from meds.object import Object
from meds.engine import Engine
from meds.event import Event
from meds.cfg import cfg

from meds.core import fleet, launcher, kernel

import logging
import select
import sys
import os

class Bot(Engine):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.channels = []

    def announce(self, txt):
        if not self.channels:
            self.out(txt)
        for channel in self.channels:
            self.say(channel, txt)

    def cmnd(self, txt):
        event = Event()
        event.btype = sname(self)
        event.origin = "user@bot"
        event.txt = txt
        kernel.put(event)
        return event

    def join(self, channel): pass

    def joinall(self):
        for channel in self.channels:
            self.join(channel)

    def out(self, txt): print(txt)

    def prompt(self):
        if cfg.main.colors: txt = "%s>%s " % (YELLOW, ENDC)
        else: txt = "> "
        sys.stdout.write(txt)
        sys.stdout.flush()

    def run_forever(self):
        self.start()
        if not cfg.main.shell:
            if cfg.main.args:
                event = self.cmnd(" ".join(cfg.main.args))
                event.wait()
        else:
            self.prompt()
            self.wait()

    def say(self, channel, txt): self.out(txt)

    def start(self):
        if self not in fleet: fleet.append(self)
        self._connected.ready()
