# meds/engine
#
#

"""
    An engine is a epoll driven event handler.

"""

from meds.utils.trace import get_exception
from meds.scheduler import Scheduler
from meds.errors import EDISCONNECT
from meds.utils.name import sname
from meds.object import Object

import threading
import select
import time
import sys

class Engine(Scheduler):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._connected = Object()
        self._poll = select.epoll()
        self._resume = Object()
        self._status = "running"
        self._starttime = time.time()
        thr = self.launch(self._start, name="%s.engine" % sname(self))
        self._thrs.append(thr)

    def _start(self):
        self._connected.wait()
        while self._status:
            for event in self.events():
                self.put(event)

    def event(self): pass

    def events(self):
        for (fd, events) in self._poll.poll():
            yield self.event()

    def register_fd(self, f):
        try: fd = f.fileno()
        except: fd = f
        self._poll.register(fd)
        self._resume.fd = fd
        return fd

    def resume(self):
        self._poll = select.epoll.fromfd(self._resume.fd)
