# meds/launcher.py
#
#

""" launches threads and provides code to wait for completion.  """

from meds.utils.name import name as thrname
from meds.utils.trace import get_exception
from meds.object import Object
from meds.tasks import Task

import threading
import logging
import queue
import time

class Launcher(Object):

    cc = "!"
    default = ""
    
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._status = "init"
        self.tasks = []
        
    def waiter(self, thrs, timeout=60.0):
        result = []
        for thr in thrs:
            logging.info("! wait for %s" % thrname(thr))
            res = thr.join(timeout)
            result.append(res)
        return result

    def launch(self, *args, **kwargs):
        name = kwargs.get("name") or thrname(args[0])
        logging.info("! launch %s" % name)
        t = Task(**kwargs)
        t.once = True
        t.name = name
        t.start()
        t.put(*args)
        return t

    def kill(self, name):
        thrs = []
        for thr in self.running(name):
            if "exit" in dir(thr): thr.exit()
            if "stop" in dir(thr): thr.stop()
            if "cancel" in dir(thr): thr.cancel() 
            logging.info("! killed %s" % thrname(thr))
            thrs.append(thr)
        return thrs

    def running(self, name=""):
        res = []
        for thr in threading.enumerate():
            if str(thr).startswith("<_"): continue
            if name and name.upper() not in str(thr).upper(): continue
            res.append(thr)
        return res
