# meds/scheduler.py
#
#

"""
     the schaeduler. 

"""

from meds.utils.trace import get_strace, get_exception
from meds.utils.getters import slice
from meds.errors import EDISCONNECT
from meds.launcher import Launcher
from meds.utils.name import sname
from meds.object import Object
from meds.ool import OOL
from meds.cfg import cfg

import importlib
import threading
import pkgutil
import logging
import queue
import types
import time
import sys

class Scheduler(Launcher):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._connected = Object()
        self._queue = queue.Queue()
        self._cbs = OOL()
        self._table = Object()
        self._handlers = OOL()
        self._status = "running"
        self._thrs = []
        thr = self.launch(self.scheduling, name="%s.schedule" % sname(self))
        self._thrs.append(thr)

    def call_cb(self, event): 
        if event.btype not in self._cbs: return
        for cb in self._cbs[event.btype]:
            logging.info("callback %s" % str(cb))
            cb(event)
             
    def dispatch(self, event):
        try: return self._handlers[event.cmnd](event)
        except KeyError: pass

    def direct(self, name, package=None): return importlib.import_module(name, package)

    def load(self, modname):
        logging.warn("# load %s" % modname)
        mod = self.direct(modname)
        self._table[modname] = mod
        return mod

    def loading(self, modname, force=False):
        from meds.core import names
        mod = self.direct(modname)
        if force: self._table[modname] = mod
        for key in dir(mod):
            obj = getattr(mod, key, None)
            if obj and type(obj) == types.FunctionType:
                if "event" in obj.__code__.co_varnames:
                    if key.startswith("cb_"): 
                        logging.warn("# register %s" % str(obj))
                        k = key.split("cb_")[-1]
                        self._cbs.register(k, obj)
                    else:
                        names.register(key, modname)
                        self._handlers.register(key, obj)

    def scheduling(self, timeout=None):
        while self._status:
            try: event = self._queue.get(timeout=timeout)
            except queue.Empty: break
            if not event: break
            try: self.handle_event(event)
            except: logging.error(get_exception())
            event.ready()

    def handle_event(self, event):
        event.parse()
        self.dispatch(event)
        self.call_cb(event)
        logging.debug(event)
        if cfg.main.shell: self.prompt()

    def modules(self, packagename="", want=""):
        if not packagename: packagename="meds"
        package = self.direct(packagename)
        for pkg in pkgutil.walk_packages(package.__path__, packagename + "."):
            yield pkg[1]

    def prompt(self): pass

    def put(self, event): self._queue.put_nowait(event)

    def reload(self, name, force=False):
        from meds.event import Event
        e = Event()
        if name not in self._table: self.load(name)
        n = name.split(".")[-1]
        if name in self._table and "shutdown" in dir(self._table[name]): self._table[name].shutdown(e) ; e.ready()
        if name not in self._table or force: self.load(name)
        else: self._table[name] = importlib.reload(self._table[name])
        if "init" in dir(self._table[name]): self._table[name].init(e) ; e.ready()
        return self._table[name]

    def stop(self):
        self._status = ""
        self._queue.put_nowait(None)
        for thr in self._thrs: thr.exit()

    def walk(self, name, force=False):
        mods = []
        for modname in sorted(list(self.modules(name))):
            self.loading(modname, force)
            mods.append(modname)
        return mods
