# meds/launcher.py
#
#

""" launches threads and provides code to wait for completion.  """

from meds.utils.name import name as thrname
from meds.utils.trace import get_exception, get_strace
from meds.object import Object

import threading
import logging
import queue
import time

class Task(threading.Thread):

    def __init__(self, *args, **kwargs):
        """
            Task(threading.Thread)

            A Task is the working unit of the bot. It extends a thread so it can get it's job passed on to it through a queue.

            >>> from meds.launcher import Task
            >>> task = Task
            >>> print(task)
            <class 'meds.tasks.Task'>

        """
        super().__init__(*args)
        self.setDaemon(kwargs.get("daemon", True))
        self._last = time.time()
        self._name = ""
        self._queue = queue.Queue()
        self._ready = threading.Event()
        self._result = ""
        self._starttime = time.time()
        self._status = "start"
        self.args = args
        self.kwargs = kwargs
        self.once = True
        self.type = "Task"

    def __del__(self):
        self._status = ""

    def isSet(self):
        return self._ready.isSet()

    def ready(self):
        logging.debug("! ready %s" % repr(self))
        self._ready.set()

    def clear(self):
        logging.debug("! clear %s" % repr(self))
        self._ready.clear()

    def wait(self, sec=180.0):
        logging.debug("! wait %s %s" % (repr(self), self._ready.wait(sec)))

    def join(self, *args, **kwargs):
        logging.debug("! join %s" % repr(self))
        super().join(*args, **kwargs)
        return self._result

    def put(self, func, *args):
        self._queue.put_nowait((func, args))
        return self

    def run(self):
        from meds.event import Event
        self._status = "working"
        while self._status != "stopped":
            _func, args = self._queue.get()
            if not self._status or not _func: break
            self._name = self.name or thrname(_func)
            self.setName(self._name)
            self._begin = time.time()
            try: self._result =  _func(*args)
            except: logging.error(get_exception())
            self._last = time.time()
            if len(args) and type(args[0]) == type(Event()): args[0].ready()
            if self.once: break
        self.status = "stopped"
        return self._result

    def exit(self):
        self._status = "stopped"
        self.put(None, None)

