# meds/user.py
#
#

""" Users object to manage bot users permissions. """

from meds.storage import Storage
from meds.object import Object

class User(Object):

    """
        User class to represent a bot user. 

        construct a user with an origin and a list of permissions

        >>> from meds.users import User
        >>> user = User("root@shell", ["REBOOT", "START"])
        >>> print(user)
        {
            "permissions": [
                "REBOOT",
                "START"
            ],
            "prefix": "user",
            "user": "root@shell"
        }
        >>> path = user.sync()

    """

    def __init__(self, origin, permissions=["USER",]):
        self.prefix = "user"
        self.user = origin
        self.permissions = permissions

class Users(Storage):

    """
        Users class to query whether a user has a permission.

        the core module contains an preconstructed users object:

        >>> from meds.core import users
        >>> users.allowed("root@shell", "REBOOT")
        True

    """

    def allowed(self, origin, perm):
        """ check whether a user has a permission. """ 
        user = self.last("user", origin)
        if user and perm.upper() in user.permissions: return True
        return False
